<?php
/*
Plugin Name: Compatibility Notes
Plugin URI: http://w-shadow.com/blog/2008/12/04/plugin-compatibility-notes/
Description: Adds compatibility information to each plugin's description. This only works for plugins that provide this info in the readme file. 
Version: 1.1
Author: Janis Elsts
Author URI: http://w-shadow.com/blog/
*/

/*
Created by Janis Elsts (email : whiteshadow@w-shadow.com) 
It's GPL.
*/

//Load the "framework"
require 'shadow_plugin_framework.php';

class CompatibilityNotes extends CompNotes_ShadowPluginFramework {
	
	function __construct(){
		//Set some plugin-specific options
		$this->magic_hooks = true;
		//Call the default constructor
		parent::__construct(__FILE__);
		
		//Load jQuery on the "Installed plugins" page
		if ( strpos($_SERVER['REQUEST_URI'], 'plugins.php')==false ) return;
			wp_enqueue_script('jquery');
        
	}
	
	function hook_admin_footer(){
		global $wp_version;

		$plugins = $this->get_plugins_compatibility();
		
		//check that this is the "Installed plugins" page
		if ( strpos($_SERVER['REQUEST_URI'], 'plugins.php')==false ) return;
		
		//Build a compatibility note for each plugin
		$notes = array();
		foreach ($plugins as $plugin_file => $plugin){
			$note = '';

			if (!empty($plugin['Requires at least']) && !empty($plugin['Tested up to'])) {
				$note = sprintf(' For WordPress %s - %s ', $plugin['Requires at least'], $plugin['Tested up to']);
			} else if ( !empty($plugin['Requires at least']) ){
				$note = sprintf(' For WordPress %s+ ', $plugin['Requires at least']);
			} else if ( !empty($plugin['Tested up to']) ){
				$note = sprintf(' Tested up to WordPress %s ', $plugin['Tested up to']);
			}
			
			/*//Show a warning if the plugin hasn't been tested with this version of WordPress
			if (!empty($plugin['Tested up to'])) {
				
				if (version_compare($wp_version, $plugin['Tested up to'], '>'))
					$note .= ". <b>Potentially incompatible with WP $wp_version</b>";
			}
			//*/
			
			if (!empty($note)) {
				$notes[$plugin_file] = $note;
			}
		}
		
		$notes_js = json_encode($notes);
		?>
<script type='text/javascript'>

var plugin_compatibility = <?php echo $notes_js; ?>;

(function ($){
	$(document).ready(function(){
		$('.plugins .second').each(function (x) {
<?php	if ( function_exists('register_widget') ){ /* WP 2.8 */  ?>

			//Find a link that mentions the plugin's filename (usually Activate/Deactivate)
			plugin_file = $(this).find('.plugin-title a.edit:first').attr('href');
			
			//Extract the filename
			re = /[&\?](?:file=|plugin=)(.+?)($|&)/i
			matches = re.exec(plugin_file);
			if (!matches) return true;
			plugin_file = unescape(matches[1]);
			
			//Add a note if one is defined
			if (plugin_compatibility[plugin_file]) {
				$(this).find('.desc a:last').after( ' | ' + plugin_compatibility[plugin_file] );
			}
			
<?php	} else { /* WP 2.5 - 2.7.1 */ ?>
			
			//Find a link that mentions the plugin's filename (usually Activate/Deactivate)
			plugin_file = $(this).find('.action-links a[href*="file="]:first').attr('href');
			
			//Extract the filename
			re = /[&\?]file=(.+?)($|&)/i
			matches = re.exec(plugin_file);
			if (!matches) return true;
			plugin_file = matches[1];
			
			//Add a note, if one is defined
			if (plugin_compatibility[plugin_file])
				$(this).find('.desc p').after('<p>'+plugin_compatibility[plugin_file]+'</p>');
<?php	}	?>
		});
	});
})(jQuery);
</script>
		<?php
	}
	
  /**
   * CompatibilityNotes::parse_plugin_readme()
   * Extracts data fields from a plugin readme that conforms to wordpress.org standards. 
   *
   * @param string $filename Path to the readme.txt file.
   * @return array or FALSE on error
   */
	function parse_plugin_readme($filename){
		$text = file_get_contents($filename);
		if (empty($text)) return false;
		
		$names = array('Contributors'=>'', 'Tags'=>'', 'Requires at least'=>'', 'Tested up to'=> '', 
			'Stable tag' => '');
		
		if ( preg_match_all('/^('.implode('|', array_keys($names)).')\s*:\s*(.*)$/mi', 
			 	$text, $matches, PREG_SET_ORDER)	)
		{
			foreach($matches as $match){
				$key = $match[1];//strtolower($match[1]);
				if ( empty($names[$key]) ){
					$names[$match[1]] = trim($match[2]);
				}
			}
		}
		
		return $names;
	}
	
  /**
   * CompatibilityNotes::get_plugins_compatibility()
   * Load compatibility information for all installed plugins and merge it with other plugin info
   *
   * @return array Comprehensive info about all installed plugins
   */
	function get_plugins_compatibility(){
		//Try to load from cache
		if ( $plugins = wp_cache_get('plugin_compatibility') ) {
			return $plugins;
		}
		
		$plugins = get_plugins();
		foreach ($plugins as $plugin_file => &$plugin){
			//Get the full path to the plugin's directory
			$plugin_folder = dirname(WP_PLUGIN_DIR . '/' . $plugin_file);
			if (file_exists($plugin_folder . '/readme.txt')){
				//Read the standard fields, if present
				if ( $data = $this->parse_plugin_readme($plugin_folder . '/readme.txt') ) {
					$plugin = array_merge($plugin, $data);
				} 
			}
		}
		
		//Cache the result for some reason
		wp_cache_set('plugin_compatibility', $plugins);
		
		return $plugins;
	}
	
  	
} //class

if ( is_admin() )
	$compatibility_notes = new CompatibilityNotes();

?>